<?php

namespace Drupal\cmlexchange\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\file\Entity\File;

/**
 * CmlFile.
 */
class FileSave extends ControllerBase {

  /**
   * Save file (init function).
   */
  public function file($content, $filename, $cml_id, $type) {
    $this->cml_id = $cml_id;
    $this->cml = \Drupal::entityManager()->getStorage('cml')->load($cml_id);
    $this->type = $type;
    $this->content = $content;
    if (strpos($filename, '.xml')) {
      // 'import.xml', 'offers.xml'.
      $file = $this->saveXml($filename);
    }
    elseif (strpos($filename, '.zip')) {
      $file = $this->saveZip($filename);
    }
    else {
      $file = $this->saveImage($filename);
    }
    return $file;
  }

  /**
   * Save XML.
   */
  public function saveXml($filename) {
    $cml = $this->cml;
    $filepath = self::cmlDir($cml);
    $file = $this->saveFile($filepath, $filename);
    if ($file->id()) {
      $file->display = 1;
      $xmlfiles = $cml->field_file->getValue();
      $xmlfiles[] = ['target_id' => $file->id()];
      $cml->field_file->setValue($xmlfiles);
      $cml->save();
    }
    else {
      $file = FALSE;
    }
    return $file;
  }

  /**
   * Save Image.
   */
  public function saveImage($filename) {
    $path = explode('/', $filename);
    $filename = array_pop($path);
    $filepath = implode('/', $path);
    $file = $this->saveFile($filepath, $filename);
    if (!$file->id()) {
      $file = FALSE;
    }
    return $file;
  }

  /**
   * Save Zip.
   */
  public function saveZip($filename) {
    $cml = $this->cml;
    $filepath = self::cmlDir($cml);
    $config = \Drupal::config('cmlexchange.settings');
    $dir = 'cml-files';
    if ($config->get('file-path')) {
      $dir = $config->get('file-path');
    }
    $filepath = "public://{$dir}/{$filepath}/";
    // Fist time save.
    if ($cml->getState() != 'zip') {
      file_prepare_directory($filepath, FILE_CREATE_DIRECTORY);
      $uri = file_unmanaged_save_data($this->content, "{$filepath}{$filename}", FILE_EXISTS_REPLACE);
      $file = File::create([
        'uri' => $uri,
        'uid' => \Drupal::currentUser()->id(),
        'filename' => $filename,
        'status' => FILE_STATUS_PERMANENT,
      ]);
      $file->save();
      if ($file->id()) {
        $xmlfiles = $cml->field_file->getValue();
        $xmlfiles[] = ['target_id' => $file->id()];
        $cml->field_file->setValue($xmlfiles);
      }
      $cml->setState('zip');
      $cml->save();
    }
    // APPEND.
    else {
      $path = \Drupal::service('file_system')->realpath("{$filepath}{$filename}");
      file_put_contents($path, $this->content, FILE_APPEND);
    }
    return TRUE;
  }

  /**
   * Save File.
   */
  public function saveFile($filepath, $filename) {
    $config = \Drupal::config('cmlexchange.settings');
    $dir = 'cml-files';
    if ($config->get('file-path')) {
      $dir = $config->get('file-path');
    }
    $filepath = "public://{$dir}/{$filepath}/";
    file_prepare_directory($filepath, FILE_CREATE_DIRECTORY);
    $uri = file_unmanaged_save_data($this->content, "{$filepath}{$filename}", FILE_EXISTS_REPLACE);
    $file = File::create([
      'uri' => $uri,
      'uid' => \Drupal::currentUser()->id(),
      'filename' => $filename,
      'status' => FILE_STATUS_PERMANENT,
    ]);
    $existing_files = entity_load_multiple_by_properties('file', ['uri' => $uri]);
    if (count($existing_files)) {
      $existing = reset($existing_files);
      $file->fid = $existing->id();
      $file->setOriginalId($existing->id());
    }
    $file->save();
    return $file;
  }

  /**
   * Get cml_id dir.
   */
  public static function cmlDir($cml) {
    $type = $cml->type->value;
    $time = format_date($cml->created->value, 'custom', 'Y-m-d--H-i-s');
    $key = substr($cml->uuid->value, 0, 8);
    $cml_id = $cml->id();
    $dir = "{$type}/{$time}-$key-{$cml_id}";
    return $dir;
  }

}
